#include <signal.h>
#include <time.h>
#include <pthread.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/syscall.h>
/* These defines come from asm-generic/siginfo.h, are necessary, but are not
   included by signal.h. I don't understand why... */
#define sigev_notify_function		_sigev_un._sigev_thread._function
#define sigev_notify_attributes	_sigev_un._sigev_thread._attribute
#define sigev_notify_thread_id	_sigev_un._tid

#define PERIOD_SIG      SIGRTMIN
#define PERIOD_SEC      0
#define PERIOD_NSEC     1000000 // 1ms


static pid_t mygettid(void)
{
	return syscall(SYS_gettid);
}

void *thread(void *arg)
{
	sigset_t mask;
	timer_t timerid;
	struct itimerspec its;
	struct sigevent evp = {
		.sigev_notify = SIGEV_THREAD_ID,
		.sigev_signo = PERIOD_SIG,
		.sigev_notify_thread_id = mygettid(),
	};
	siginfo_t si;
	int sig;

	/* Bloque le signal PERIOD_SIG pour le thread appelant uniquement */
	if (sigemptyset(&mask) ||
	    sigaddset(&mask, PERIOD_SIG) ||
	    sigprocmask(SIG_BLOCK, &mask, NULL))
		pthread_exit((void *) 10);

	if (timer_create(CLOCK_REALTIME, &evp, &timerid))
		pthread_exit((void *) 11);

	its.it_value.tv_sec = PERIOD_SEC;
	its.it_value.tv_nsec = PERIOD_NSEC;
	its.it_interval.tv_sec = PERIOD_SEC;
	its.it_interval.tv_nsec = PERIOD_NSEC;
	if (timer_settime(timerid, 0, &its, NULL))
		pthread_exit((void *) 12);

	while (1)
	{
		if ((sig = sigwaitinfo(&mask, &si)) < 0)
			pthread_exit((void *) 13);
		if (sig != PERIOD_SIG)
			continue;
		/* Periodic task... */
	}

	pthread_exit((void *) 0);
}

int main(int argc, char *argv[])
{
	pthread_t th;
	int ret;

	if (pthread_create(&th, NULL, thread, NULL))
		return 2;

	pthread_join(th, (void *) &ret);
	return ret;
}
